@@ -575,6 +575,14 @@ class SqlWalker
         return implode(', ', $sqlParts);
     }
 
+    /**
+     * Walks down an EntityAsDtoArgumentExpression AST node, thereby generating the appropriate SQL.
+     */
+    public function walkEntityAsDtoArgumentExpression(AST\EntityAsDtoArgumentExpression $expr): string
+    {
+        return implode(', ', $this->walkObjectExpression($expr->expression, [], $expr->identificationVariable ?: null));
+    }
+
     /**
      * Walks down an IdentificationVariable (no AST node associated), thereby generating the SQL.
      */
@@ -1356,32 +1364,79 @@ class SqlWalker
                     $partialFieldSet = [];
                 }
 
-                $class       = $this->getMetadataForDqlAlias($dqlAlias);
-                $resultAlias = $selectExpression->fieldIdentificationVariable ?: null;
+                $sql .= implode(', ', $this->walkObjectExpression($dqlAlias, $partialFieldSet, $selectExpression->fieldIdentificationVariable ?: null));
+        }
 
-                if (! isset($this->selectedClasses[$dqlAlias])) {
-                    $this->selectedClasses[$dqlAlias] = [
-                        'class'       => $class,
-                        'dqlAlias'    => $dqlAlias,
-                        'resultAlias' => $resultAlias,
-                    ];
-                }
+        return $sql;
+    }
 
-                $sqlParts = [];
+    /**
+     * Walks down an Object Expression AST node and return Sql Parts
+     *
+     * @param mixed[] $partialFieldSet
+     *
+     * @return string[]
+     */
+    public function walkObjectExpression(string $dqlAlias, array $partialFieldSet, string|null $resultAlias): array
+    {
+        $class = $this->getMetadataForDqlAlias($dqlAlias);
 
-                // Select all fields from the queried class
-                foreach ($class->fieldMappings as $fieldName => $mapping) {
-                    if ($partialFieldSet && ! in_array($fieldName, $partialFieldSet, true)) {
+        if (! isset($this->selectedClasses[$dqlAlias])) {
+            $this->selectedClasses[$dqlAlias] = [
+                'class'       => $class,
+                'dqlAlias'    => $dqlAlias,
+                'resultAlias' => $resultAlias,
+            ];
+        }
+
+        $sqlParts = [];
+
+        // Select all fields from the queried class
+        foreach ($class->fieldMappings as $fieldName => $mapping) {
+            if ($partialFieldSet && ! in_array($fieldName, $partialFieldSet, true)) {
+                continue;
+            }
+
+            $tableName = isset($mapping->inherited)
+                ? $this->em->getClassMetadata($mapping->inherited)->getTableName()
+                : $class->getTableName();
+
+            $sqlTableAlias    = $this->getSQLTableAlias($tableName, $dqlAlias);
+            $columnAlias      = $this->getSQLColumnAlias($mapping->columnName);
+            $quotedColumnName = $this->quoteStrategy->getColumnName($fieldName, $class, $this->platform);
+
+            $col = $sqlTableAlias . '.' . $quotedColumnName;
+
+            $type = Type::getType($mapping->type);
+            $col  = $type->convertToPHPValueSQL($col, $this->platform);
+
+            $sqlParts[] = $col . ' AS ' . $columnAlias;
+
+            $this->scalarResultAliasMap[$resultAlias][] = $columnAlias;
+
+            $this->rsm->addFieldResult($dqlAlias, $columnAlias, $fieldName, $class->name);
+
+            if (! empty($mapping->enumType)) {
+                $this->rsm->addEnumResult($columnAlias, $mapping->enumType);
+            }
+        }
+
+        // Add any additional fields of subclasses (excluding inherited fields)
+        // 1) on Single Table Inheritance: always, since its marginal overhead
+        // 2) on Class Table Inheritance only if partial objects are disallowed,
+        //    since it requires outer joining subtables.
+        if ($class->isInheritanceTypeSingleTable() || ! $this->query->getHint(Query::HINT_FORCE_PARTIAL_LOAD)) {
+            foreach ($class->subClasses as $subClassName) {
+                $subClass      = $this->em->getClassMetadata($subClassName);
+                $sqlTableAlias = $this->getSQLTableAlias($subClass->getTableName(), $dqlAlias);
+
+                foreach ($subClass->fieldMappings as $fieldName => $mapping) {
+                    if (isset($mapping->inherited) || ($partialFieldSet && ! in_array($fieldName, $partialFieldSet, true))) {
                         continue;
                     }
 
-                    $tableName = isset($mapping->inherited)
-                        ? $this->em->getClassMetadata($mapping->inherited)->getTableName()
-                        : $class->getTableName();
-
-                    $sqlTableAlias    = $this->getSQLTableAlias($tableName, $dqlAlias);
                     $columnAlias      = $this->getSQLColumnAlias($mapping->columnName);
-                    $quotedColumnName = $this->quoteStrategy->getColumnName($fieldName, $class, $this->platform);
+                    $quotedColumnName = $this->quoteStrategy->getColumnName($fieldName, $subClass, $this->platform);
 
                     $col = $sqlTableAlias . '.' . $quotedColumnName;
 
@@ -1392,48 +1447,12 @@ class SqlWalker
 
                     $this->scalarResultAliasMap[$resultAlias][] = $columnAlias;
 
-                    $this->rsm->addFieldResult($dqlAlias, $columnAlias, $fieldName, $class->name);
-
-                    if (! empty($mapping->enumType)) {
-                        $this->rsm->addEnumResult($columnAlias, $mapping->enumType);
-                    }
+                    $this->rsm->addFieldResult($dqlAlias, $columnAlias, $fieldName, $subClassName);
                 }
-
-                // Add any additional fields of subclasses (excluding inherited fields)
-                // 1) on Single Table Inheritance: always, since its marginal overhead
-                // 2) on Class Table Inheritance only if partial objects are disallowed,
-                //    since it requires outer joining subtables.
-                if ($class->isInheritanceTypeSingleTable() || ! $this->query->getHint(Query::HINT_FORCE_PARTIAL_LOAD)) {
-                    foreach ($class->subClasses as $subClassName) {
-                        $subClass      = $this->em->getClassMetadata($subClassName);
-                        $sqlTableAlias = $this->getSQLTableAlias($subClass->getTableName(), $dqlAlias);
-
-                        foreach ($subClass->fieldMappings as $fieldName => $mapping) {
-                            if (isset($mapping->inherited) || ($partialFieldSet && ! in_array($fieldName, $partialFieldSet, true))) {
-                                continue;
-                            }
-
-                            $columnAlias      = $this->getSQLColumnAlias($mapping->columnName);
-                            $quotedColumnName = $this->quoteStrategy->getColumnName($fieldName, $subClass, $this->platform);
-
-                            $col = $sqlTableAlias . '.' . $quotedColumnName;
-
-                            $type = Type::getType($mapping->type);
-                            $col  = $type->convertToPHPValueSQL($col, $this->platform);
-
-                            $sqlParts[] = $col . ' AS ' . $columnAlias;
-
-                            $this->scalarResultAliasMap[$resultAlias][] = $columnAlias;
-
-                            $this->rsm->addFieldResult($dqlAlias, $columnAlias, $fieldName, $subClassName);
-                        }
-                    }
-                }
-
-                $sql .= implode(', ', $sqlParts);
+            }
         }
 
-        return $sql;
+        return $sqlParts;
     }
 
     public function walkQuantifiedExpression(AST\QuantifiedExpression $qExpr): string
@@ -1499,11 +1518,17 @@ class SqlWalker
     {
         $sqlSelectExpressions = [];
         $objIndex             = $newObjectResultAlias ?: $this->newObjectCounter++;
+        $aliasGap             = $newObjectExpression->hasNamedArgs ? null : 0;
 
         foreach ($newObjectExpression->args as $argIndex => $e) {
-            $resultAlias = $this->scalarResultCounter++;
-            $columnAlias = $this->getSQLColumnAlias('sclr');
-            $fieldType   = 'string';
+            if (! $newObjectExpression->hasNamedArgs) {
+                $argIndex += $aliasGap;
+            }
+
+            $resultAlias    = $this->scalarResultCounter++;
+            $columnAlias    = $this->getSQLColumnAlias('sclr');
+            $fieldType      = 'string';
+            $isScalarResult = true;
 
             switch (true) {
                 case $e instanceof AST\NewObjectExpression:
@@ -1549,18 +1574,34 @@ class SqlWalker
                     $sqlSelectExpressions[] = trim($e->dispatch($this)) . ' AS ' . $columnAlias;
                     break;
 
+                case $e instanceof AST\EntityAsDtoArgumentExpression:
+                    $alias                                             = $e->identificationVariable ?: $columnAlias;
+                    $this->rsm->nestedNewObjectArguments[$columnAlias] = ['ownerIndex' => $objIndex, 'argIndex' => $argIndex, 'argAlias' => $alias];
+                    $this->rsm->nestedEntities[$alias]                 = ['parent' => $objIndex, 'argIndex' => $argIndex, 'type' => 'entity'];
+
+                    $sqlSelectExpressions[] = trim($e->dispatch($this));
+                    break;
+
+                case $e instanceof AST\AllFieldsExpression:
+                    $isScalarResult         = false;
+                    $sqlSelectExpressions[] = $e->dispatch($this, $objIndex, $argIndex, $aliasGap);
+                    break;
+
                 default:
                     $sqlSelectExpressions[] = trim($e->dispatch($this)) . ' AS ' . $columnAlias;
                     break;
             }
 
-            $this->scalarResultAliasMap[$resultAlias] = $columnAlias;
-            $this->rsm->addScalarResult($columnAlias, $resultAlias, $fieldType);
+            if ($isScalarResult) {
+                $this->scalarResultAliasMap[$resultAlias] = $columnAlias;
+                $this->rsm->addScalarResult($columnAlias, $resultAlias, $fieldType);
 
-            $this->rsm->newObjectMappings[$columnAlias] = [
-                'objIndex'  => $objIndex,
-                'argIndex'  => $argIndex,
-            ];
+                $this->rsm->newObjectMappings[$columnAlias] = [
+                    'className' => $newObjectExpression->className,
+                    'objIndex'  => $objIndex,
+                    'argIndex'  => $argIndex,
+                ];
+            }
         }
 
         $this->rsm->newObject[$objIndex] = $newObjectExpression->className;
@@ -2265,6 +2306,42 @@ class SqlWalker
         return $resultAlias;
     }
 
+    public function walkAllEntityFieldsExpression(AST\AllFieldsExpression $expression, int|string $objIndex, int|string $argIndex, int|null &$aliasGap): string
+    {
+        $dqlAlias = $expression->identificationVariable;
+        $class    = $this->getMetadataForDqlAlias($expression->identificationVariable);
+
+        $sqlParts = [];
+        // Select all fields from the queried class
+        foreach ($class->fieldMappings as $fieldName => $mapping) {
+            $tableName = isset($mapping->inherited)
+                ? $this->em->getClassMetadata($mapping->inherited)->getTableName()
+                : $class->getTableName();
+
+            $sqlTableAlias    = $this->getSQLTableAlias($tableName, $dqlAlias);
+            $columnAlias      = $this->getSQLColumnAlias($mapping->columnName);
+            $quotedColumnName = $this->quoteStrategy->getColumnName($fieldName, $class, $this->platform);
+
+            $col = $sqlTableAlias . '.' . $quotedColumnName;
+
+            $type = Type::getType($mapping->type);
+            $col  = $type->convertToPHPValueSQL($col, $this->platform);
+
+            $sqlParts[] = $col . ' AS ' . $columnAlias;
+
+            $this->scalarResultAliasMap[$objIndex][] = $columnAlias;
+
+            $this->rsm->addScalarResult($columnAlias, $objIndex, $mapping->type);
+
+            $this->rsm->newObjectMappings[$columnAlias] = [
+                'objIndex'  => $objIndex,
+                'argIndex'  => $aliasGap === null ? $fieldName : (int) $argIndex + $aliasGap++,
+            ];
+        }
+
+        return implode(', ', $sqlParts);
+    }
+
     /**
      * @return string The list in parentheses of valid child discriminators from the given class
      *
