@@ -32,6 +32,10 @@ class GH10808Test extends OrmFunctionalTestCase
 
     public function testDQLDeferredEagerLoad(): void
     {
+        if ($this->_em->getConfiguration()->isNativeLazyObjectsEnabled()) {
+            self::markTestSkipped('Test requires lazy loading to be disabled');
+        }
+
         $appointment = new GH10808Appointment();
 
         $this->_em->persist($appointment);
