@@ -194,6 +194,7 @@ use function strtolower;
 use function var_export;
 
 use const PHP_EOL;
+use const PHP_VERSION_ID;
 
 /**
  * Base testcase class for all functional ORM testcases.
@@ -938,6 +939,12 @@ abstract class OrmFunctionalTestCase extends OrmTestCase
             $this->isSecondLevelCacheEnabled = true;
         }
 
+        $enableNativeLazyObjects = getenv('ENABLE_NATIVE_LAZY_OBJECTS');
+
+        if (PHP_VERSION_ID >= 80400 && $enableNativeLazyObjects) {
+            $config->enableNativeLazyObjects(true);
+        }
+
         $config->setMetadataDriverImpl(
             $mappingDriver ?? new AttributeDriver([
                 realpath(__DIR__ . '/Models/Cache'),
@@ -1120,4 +1127,9 @@ abstract class OrmFunctionalTestCase extends OrmTestCase
     {
         return $this->_em->getUnitOfWork()->isUninitializedObject($entity);
     }
+
+    final protected function initializeObject(object $entity): void
+    {
+        $this->_em->getUnitOfWork()->initializeObject($entity);
+    }
 }
